SystemOrganization addCategory: #'FileSystem-Disk'!

FileSystemStore subclass: #DiskStore
	instanceVariableNames: 'maxFileNameLength'
	classVariableNames: 'CurrentFS Primitives'
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!DiskStore commentStamp: '<historical>' prior: 0!
I am an abstract superclass for disk store implementations. My subclasses provide access to the actual data storage of a particular kind of filesystem. 
!

!DiskStore class methodsFor: 'current' stamp: 'CamilloBruni 5/10/2012 15:41'!
activeClass
	self allSubclassesDo: [:ea | 
		ea isActiveClass ifTrue: [^ ea]].
	^ self! !

!DiskStore class methodsFor: 'class initialization' stamp: 'CamilloBruni 7/18/2012 13:55'!
checkVMVersion
	"Display a warning if the VM is too old"
	| displayError |
	displayError := [ ^ self inform: 'Your VM is too old for this image. Please download the latest VM.' ].
	[(VirtualMachine interpreterSourceDate > '2012-07-08+2:00' asDate)
		ifFalse: displayError
	] on: Error do: [ :e| displayError value ].! !

!DiskStore class methodsFor: 'current' stamp: 'cwp 2/27/2011 10:02'!
createDefault
	^ self new! !

!DiskStore class methodsFor: 'current' stamp: 'EstebanLorenzano 4/3/2012 11:30'!
current
	^ self currentFileSystem store! !

!DiskStore class methodsFor: 'current' stamp: 'CamilloBruni 5/13/2012 19:17'!
currentFileSystem
	^ CurrentFS ifNil: [
		CurrentFS := FileSystem store: self activeClass createDefault]! !

!DiskStore class methodsFor: 'current' stamp: 'EstebanLorenzano 4/3/2012 13:11'!
currentFilesystem
	self 
		deprecated: 'Use #currentFileSystem' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self currentFileSystem! !

!DiskStore class methodsFor: 'public' stamp: 'Cami 7/9/2012 11:12'!
delimiter
	^ self current delimiter! !

!DiskStore class methodsFor: 'class initialization' stamp: 'EstebanLorenzano 6/21/2012 12:53'!
initialize
	self useFilePlugin.
	Smalltalk 
		removeFromStartUpList: self;
		removeFromShutDownList: self;
		addToStartUpList: self after: OSPlatform;
		addToShutDownList: self! !

!DiskStore class methodsFor: 'current' stamp: 'cwp 2/18/2011 17:20'!
isActiveClass
	^ self delimiter = Primitives delimiter! !

!DiskStore class methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:03'!
maxFileNameLength
	self subclassResponsibility ! !

!DiskStore class methodsFor: 'current' stamp: 'cwp 4/4/2011 19:04'!
reset
	CurrentFS := nil! !

!DiskStore class methodsFor: 'class initialization' stamp: 'EstebanLorenzano 6/21/2012 12:52'!
shutDown: quitting
	"Force to detect filesystem after image restart"
	self reset! !

!DiskStore class methodsFor: 'class initialization' stamp: 'CamilloBruni 7/18/2012 12:22'!
startUp: resuming
	self checkVMVersion.
	resuming 
		ifTrue: [ self reset ]! !

!DiskStore class methodsFor: 'class initialization' stamp: 'EstebanLorenzano 4/2/2012 11:46'!
useFilePlugin
	Primitives := FilePluginPrims new! !

!DiskStore methodsFor: 'comparing' stamp: 'cwp 2/27/2011 09:50'!
= other
	^ self species = other species! !

!DiskStore methodsFor: 'private' stamp: 'cwp 2/18/2011 13:27'!
basenameFromEntry: entry
	^ entry at: 1! !

!DiskStore methodsFor: 'private' stamp: 'CamilloBruni 7/17/2012 18:17'!
basicCreationTime: anEntry
	" the entry contains the seconds since the squeak epoch in local time"
	^  (DateAndTime fromSeconds: (anEntry at: 2) offset: 0) translateTo: DateAndTime localOffset! !

!DiskStore methodsFor: 'public' stamp: 'EstebanLorenzano 6/20/2012 12:27'!
basicEntryAt: aPath
	| encodedPath encodedBasename |
	
	encodedPath := Primitives encode: (self stringFromPath: aPath parent).
	encodedBasename := Primitives encode: aPath basename.
	
	^ (Primitives lookupDirectory: encodedPath filename: encodedBasename)
		ifNil: [ #badDirectoryPath ].
	! !

!DiskStore methodsFor: 'private' stamp: 'cwp 2/18/2011 11:32'!
basicIsDirectory: anEntry
	^ anEntry at: 4! !

!DiskStore methodsFor: 'private' stamp: 'cwp 2/18/2011 11:33'!
basicIsFile: anEntry
	^ (anEntry at: 4) not! !

!DiskStore methodsFor: 'private' stamp: 'EstebanLorenzano 8/2/2012 16:51'!
basicIsSymlink: anEntry
	^(anEntry size >= 7)
		ifTrue: [ anEntry at: 7 ]
		ifFalse: [ false ]! !

!DiskStore methodsFor: 'private' stamp: 'CamilloBruni 7/17/2012 18:17'!
basicModificationTime: anEntry
	" the entry contains the seconds since the squeak epoch in local time"
	^ (DateAndTime fromSeconds: (anEntry at: 3) offset: 0) translateTo: DateAndTime localOffset! !

!DiskStore methodsFor: 'public' stamp: 'cwp 2/28/2011 12:35'!
basicOpen: aPath writable: aBoolean
	| string encoded |
	string := self stringFromPath: aPath.
	encoded := Primitives encode: string.
	^ Primitives open: encoded writable: aBoolean! !

!DiskStore methodsFor: 'private' stamp: 'EstebanLorenzano 8/2/2012 15:38'!
basicPosixPermissions: anEntry
	^ (anEntry size >= 6)
		ifTrue: [ anEntry at: 6 ]
		ifFalse: [ nil ].! !

!DiskStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 21:13'!
basicSize: anEntry
	^ (anEntry at: 5)! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 5/13/2012 19:17'!
checkName: aFileName fixErrors: fixErrors
	"Check a string aFileName for validity as a file name. Answer the original file name if it is valid. If the name is not valid (e.g., it is too long or contains illegal characters) and fixing is false, raise an error. If fixing is true, fix the name (usually by truncating and/or tranforming characters), and answer the corrected name. The default behavior is just to truncate the name to the maximum length for this platform. Subclasses can do any kind of checking and correction appropriate for their platform."
	
	| maxLength |
	aFileName size = 0 ifTrue: [self error: 'zero length file name'].
	maxLength := self maxFileNameLength.
	aFileName size > maxLength ifTrue: [
		fixErrors
			ifTrue: [^ aFileName contractTo: maxLength]
			ifFalse: [self error: 'file name is too long']].
	^ aFileName! !

!DiskStore methodsFor: 'public' stamp: 'cwp 4/3/2011 22:17'!
createDirectory: path
	"Create a directory for the argument path. 
	If the path refers to an existing file, raise FileExists.
	If the path refers to an existing directory, raise DirectoryExists.
	If the parent directory of the path does not exist, raise DirectoryDoesNotExist"

	| parent encodedPathString pathString result |
	pathString := self stringFromPath: path.
	encodedPathString := Primitives encode: pathString.
	result := Primitives createDirectory: encodedPathString.
	result
		ifNil: [ 
			parent := path parent.
			(self exists: path)
				ifTrue: [ 
					(self isFile: path)
						ifTrue: [ self signalFileExists: path ]
						ifFalse: [ self signalDirectoryExists: path ] ].
			(self isDirectory: parent)
				ifFalse: [ ^ self signalDirectoryDoesNotExist: parent ].
			self primitiveFailed ].
	^ self! !

!DiskStore methodsFor: 'accessing' stamp: 'cwp 2/27/2011 10:03'!
defaultWorkingDirectory
	| pathString |
	pathString := Primitives decode: Primitives imageFile.
	^ (self pathFromString: pathString) parent! !

!DiskStore methodsFor: 'public' stamp: 'cwp 2/27/2011 10:03'!
delete: path
	| pathString encodedPathString |
	pathString := self stringFromPath: path.
	encodedPathString := Primitives encode: pathString.
	(self isDirectory: path)
		ifTrue: [ Primitives deleteDirectory: encodedPathString ]
		ifFalse: [ 
			StandardFileStream
				retryWithGC: [ Primitives deleteFile: encodedPathString ]
				until: [ :result | result notNil ]
				forFileNamed: pathString ]! !

!DiskStore methodsFor: 'private' stamp: 'cwp 2/27/2011 10:03'!
directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock
	| encodedPathString index entry pathString |
	index := 1.
	pathString := self stringFromPath: aPath.
	encodedPathString := Primitives encode: pathString.
	entry := Primitives lookupEntryIn: encodedPathString index: index.
	entry = #badDirectoryPath
		ifTrue: [ ^ absentBlock value ].
	[ entry isNil ]
		whileFalse: [ 
			entry at: 1 put: (Primitives decode: entry first).
			aBlock value: entry.
			index := index + 1.
			entry := Primitives lookupEntryIn: encodedPathString index: index ].
	^ self! !

!DiskStore methodsFor: 'printing' stamp: 'SeanDeNigris 2/9/2013 09:12'!
forReferencePrintOn: aStream
	aStream nextPutAll: 'File @ '! !

!DiskStore methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/2/2012 11:46'!
handleClass
	^ FileHandle! !

!DiskStore methodsFor: 'comparing' stamp: 'cwp 2/27/2011 09:51'!
hash
	^ self species hash! !

!DiskStore methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/13/2012 19:18'!
initialize
	super initialize.
	maxFileNameLength := Smalltalk vm maxFilenameLength ifNil: [ 255 ].! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 6/1/2012 14:46'!
isDirectory: aPath
	| entry |
	
	aPath isRoot ifTrue: [ ^ true ].
	
	entry := self  basicEntryAt: aPath.
	
	^ entry == #badDirectoryPath 
		ifTrue: [  false ]
		ifFalse: [ self basicIsDirectory: entry ].
	! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 6/17/2012 17:22'!
isFile: aPath
	| entry |
	
	aPath isRoot ifTrue: [ ^ false ].
	
	entry := self  basicEntryAt: aPath.
	
	^ entry == #badDirectoryPath 
		ifTrue: [  false ]
		ifFalse: [ self basicIsFile: entry ].
	! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 00:56'!
isReadable: aPath
	(self exists: aPath) ifFalse: [ ^ false ].
	self flag: 'TODO: we need a decent primitive for this...'.
	(self basicOpen: aPath writable: false) 
		ifNotNilDo: [ :id|
			Primitives close: id.
			^ true].
	^ false! !

!DiskStore methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 8/2/2012 15:43'!
isSymlink: aPath
	| entry |
	
	aPath isRoot ifTrue: [ ^false ].
	
	entry := self  basicEntryAt: aPath.
	
	^ entry == #badDirectoryPath 
		ifTrue: [  false ]
		ifFalse: [ self basicIsSymlink: entry ].
	! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 01:01'!
isWritable: aPath
	(self exists: aPath) ifFalse: [ ^ false ].
	self flag: 'TODO: we need a decent primitive for this...'.
	(self basicOpen: aPath writable: true) 
		ifNotNilDo: [ :id|
			Primitives close: id.
			^ true].
	^ false! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 5/13/2012 19:17'!
maxFileNameLength
	^ maxFileNameLength! !

!DiskStore methodsFor: 'private' stamp: 'CamilloBruni 6/1/2012 14:44'!
nodeAt: aPath ifPresent: presentBlock ifAbsent: absentBlock
	
	| entry |
	
	aPath isRoot ifTrue: [ ^ presentBlock value: self rootNode ].
	
	entry := self basicEntryAt: aPath.
	
	^ entry == #badDirectoryPath 
		ifTrue: absentBlock
		ifFalse: [
			entry at: 1 put: aPath basename.
			presentBlock value: entry ].! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 2/20/2012 21:41'!
openFileStream: path writable: writable
	| fullPath |
	fullPath := self stringFromPath: path.
	"redirect over the default implementation"
	^ writable 
		ifFalse: [ FileStream readOnlyFileNamed: fullPath ]
		ifTrue: [ FileStream fileNamed: fullPath ]! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 1/23/2012 15:52'!
rename: sourcePath to: destinationPath

	| sourcePathString encodedSourcePathString targetPathString encodedTargetPathString result |
	sourcePathString := self stringFromPath: sourcePath.
	encodedSourcePathString := Primitives encode: sourcePathString.
	targetPathString := self stringFromPath: destinationPath.
	encodedTargetPathString := Primitives encode: targetPathString.
	^ Primitives rename: encodedSourcePathString to: encodedTargetPathString.! !

!DiskStore methodsFor: 'private' stamp: 'cwp 2/18/2011 11:37'!
rootNode
	^ #('' 0 0 true 0)! !

DiskStore subclass: #UnixStore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!UnixStore commentStamp: '<historical>' prior: 0!
I'm a specific store for Unix file systems!

UnixStore subclass: #MacStore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!MacStore commentStamp: '<historical>' prior: 0!
I'm a specific store for OSX file systems!

!MacStore class methodsFor: 'current' stamp: 'CamilloBruni 5/10/2012 15:43'!
isActiveClass
	^ Smalltalk os isMacOS! !

!MacStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 13:11'!
mimeTypesAt: aPath
	"Return a list of MIME types applicable to the receiver. This default implementation uses the file name extension to figure out what we're looking at but specific subclasses may use other means of figuring out what the type of some file is. Some systems like the macintosh use meta data on the file to indicate data type"
	"| typeCreator type | 
	typeCreator := self getMacFileTypeAndCreator: ((self fullNameFor: fileName)).
	type := (typeCreator at: 1) asLowercase.
	^TypeToMimeMappings at: type ifAbsent:[super mimeTypesFor: fileName]"
	self flag: 'TODO: properly implement the mac os version'.
	^ super mimeTypesAt: aPath! !

!UnixStore class methodsFor: 'public' stamp: 'cwp 2/18/2011 17:19'!
delimiter
	^ $/! !

!UnixStore class methodsFor: 'current' stamp: 'CamilloBruni 5/10/2012 15:43'!
isActiveClass
	^ Smalltalk os isUnix and: [ super isActiveClass ]! !

!UnixStore class methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:21'!
isCaseSensitive
	"except for OSX the default is case sensitive"
	^ OSPlatform isMacOSX not! !

!UnixStore class methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:04'!
maxFileNameLength

	^ 255! !

!UnixStore class methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:11'!
separator 
	^ $:! !

!UnixStore methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:08'!
checkName: aFileName fixErrors: fixing
	"Check if the file name contains any invalid characters"
	| fName |
	fName := super checkName: aFileName fixErrors: fixing.
	
	(fName includes: self delimiter) ifFalse:
		[^fName].
	
	fixing ifFalse: [self error:'Invalid file name'].
	
	^ fName copyReplaceAll: self delimiter asString with: '#'! !

!UnixStore methodsFor: 'public-testing' stamp: 'ThierryGoubier 7/3/2013 11:26'!
exists: aPath
	"system call equivalent: euidaccess(aPath, F_OK)"

	^ (self nbAccess: (self stringFromPath: aPath) convertToSystemString mode: 0) == 0! !

!UnixStore methodsFor: 'public-testing' stamp: 'ThierryGoubier 7/3/2013 11:21'!
isReadable: aPath
	"system call equivalent: euidaccess(aPath, R_OK)"
	
	^ (self nbAccess: (self stringFromPath: aPath) convertToSystemString mode: 4) == 0! !

!UnixStore methodsFor: 'public-testing' stamp: 'ThierryGoubier 7/3/2013 11:22'!
isWritable: aPath
	"system call equivalent: euidaccess(aPath, W_OK)"

	^ (self nbAccess: (self stringFromPath: aPath) convertToSystemString mode: 2) == 0! !

!UnixStore methodsFor: 'private' stamp: 'ThierryGoubier 7/3/2013 14:55'!
nbAccess: str mode: mode
	"
	This is a basic example for making an external call. 
	This method calls a Standard C library euidaccess() function
	
	F_OK=0, R_OK=4, W_OK=2, X_OK=1
	
	Safer to use euidaccess instead of access.
"

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: #errorCode>
	^ self nbCall: #(#int #euidaccess #(#String #_1 #, #int #_2)) module: NativeBoost CLibrary! !

DiskStore subclass: #WindowsStore
	instanceVariableNames: 'disk'
	classVariableNames: 'Disks'
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!WindowsStore commentStamp: '<historical>' prior: 0!
I'm a specific store for Windows file systems!

!WindowsStore class methodsFor: 'accessing' stamp: 'cwp 2/18/2011 17:19'!
delimiter
	^ $\! !

!WindowsStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:20'!
isCaseSensitive
	^ false! !

!WindowsStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/10/2012 16:05'!
maxFileNameLength
	self flag: 'TODO: more tests needed here!!'.
	^ 255! !

!WindowsStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:11'!
separator 
	^ $\! !

!WindowsStore methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:07'!
checkName: aFileName fixErrors: fixing
	"Check if the file name contains any invalid characters"
	
	| fName badChars hasBadChars |
	fName := super checkName: aFileName fixErrors: fixing.
	badChars := (#( $: $< $> $| $/ $\ $? $* $"), ((0 to: 31) collect: [:n | n asCharacter])) asSet.
	
	(fName includesAnyOf: badChars)
		ifFalse: [^ fName].
	
	fixing ifFalse: [^self error: 'filename contains invalid characters'].
	
	^ fName collect:
		[:char | (badChars includes: char) 
				ifTrue: [$#] 
				ifFalse: [char]]! !

!WindowsStore methodsFor: 'converting' stamp: 'CamilloBruni 7/6/2012 15:21'!
pathFromString: aString
	| normalized pathClass |
	normalized := aString copy replaceAll: UnixStore delimiter with: self delimiter.
	pathClass := (Path isAbsoluteWindowsPath: normalized)
		ifTrue: [ AbsolutePath ]
		ifFalse: [ RelativePath ].
	^pathClass withAll: (pathClass canonicalizeElements: (self delimiter split: normalized))! !

!WindowsStore methodsFor: 'converting' stamp: 'cwp 2/28/2011 12:32'!
printPath: aPath on: aStream
	aPath printOn: aStream delimiter: self delimiter! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/4/2012 19:19'!
* aFileOrDirectoryName
	^ self disk * aFileOrDirectoryName! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/4/2012 19:19'!
/ aFileOrDirectoryName
	^ self disk / aFileOrDirectoryName! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'EstebanLorenzano 4/3/2012 11:30'!
disk
	"Answer a filesystem that represents the 'on-disk' filesystem used by the host operating system."

	^ DiskStore currentFileSystem! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/7/2012 11:57'!
lookInUsualPlaces: fileName
	"Check the default directory, the imagePath, and the vmPath (and the vmPath's owner) for this file."

	| file |
	 (file := fileName asFileReference) exists
		ifTrue: [ ^ file ].

	(file := Smalltalk imageFile resolve: fileName) exists
		ifTrue: [ ^ file ].

	(file :=  Smalltalk vmDirectory resolve: fileName) exists
		ifTrue: [ ^ file ].

	(file := Smalltalk vmDirectory parent resolve: fileName) exists
		ifTrue: [ ^ file ].

	^ nil! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'EstebanLorenzano 4/3/2012 13:13'!
onDisk
	"This method provides compatibility with the original Filesystem interface"
	self deprecated: 'Use #disk' on: '3 April 2012' in: 'Pharo 1.4'. 
	^ self disk! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/4/2012 20:54'!
root
	^ self disk root! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/4/2012 20:53'!
workingDirectory
	^ self disk workingDirectory! !

Object subclass: #FilePluginPrims
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!FilePluginPrims commentStamp: 'cwp 11/18/2009 13:02' prior: 0!
I provide an interface to the primitives in the FilePlugin.
!

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:26'!
atEnd: id
	"Answer true if the file position is at the end of the file."

	<primitive: 'primitiveFileAtEnd' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:27'!
close: id
	"Close this file."

	<primitive: 'primitiveFileClose' module: 'FilePlugin'>
	! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 11/17/2009 16:26'!
createDirectory: fullPath
	"Create a directory named by the given path. 
	Fail if the path is bad or if a file or directory by that name already exists."

 	<primitive: 'primitiveDirectoryCreate' module: 'FilePlugin'>
	^ nil
! !

!FilePluginPrims methodsFor: 'encoding primitives' stamp: 'lr 7/13/2010 14:10'!
decode: aString
	^ aString convertFromWithConverter: LanguageEnvironment defaultFileNameConverter! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:21'!
deleteDirectory: fullPath
	"Delete the directory named by the given path. 
	Fail if the path is bad or if a directory by that name does not exist."

 	<primitive: 'primitiveDirectoryDelete' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:22'!
deleteFile: aFileName
	"Delete the file of the given name. 
	Return self if the primitive succeeds, nil otherwise."

	<primitive: 'primitiveFileDelete' module: 'FilePlugin'>
	^ nil
! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 10/11/2009 11:02'!
delimiter
	"Return the path delimiter for the underlying platform's file system."

 	<primitive: 'primitiveDirectoryDelimitor' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'encoding primitives' stamp: 'lr 7/13/2010 14:11'!
encode: aString
	^ aString convertToWithConverter: LanguageEnvironment defaultFileNameConverter! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'lr 3/21/2010 12:10'!
flush: id
	"Flush pending changes to the disk"
	
	| pos |
	<primitive: 'primitiveFileFlush' module: 'FilePlugin'>
	"In some OS's seeking to 0 and back will do a flush"
	pos := self getPosition: id.
	self setPosition: id to: 0; setPosition: id to: pos! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:23'!
getMacFile: fileName type: typeString creator: creatorString
	"Get the Macintosh file type and creator info for the file with the given name. Fails if the file does not exist or if the type and creator type arguments are not strings of length 4. This primitive is Mac specific; it is a noop on other platforms."

 	<primitive: 'primitiveDirectoryGetMacTypeAndCreator' module: 'FilePlugin'>

! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:27'!
getPosition: id
	"Get this files current position."

	<primitive: 'primitiveFileGetPosition' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 10/11/2009 11:02'!
imageFile
	"Answer the full path name for the current image."

	<primitive: 121>
	self primitiveFailed! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'CamilloBruni 1/17/2012 17:09'!
lookupDirectory: fullPath filename: fileName
 	<primitive: 'primitiveDirectoryEntry' module: 'FilePlugin'>
	^ #badDirectoryPath

! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:24'!
lookupEntryIn: fullPath index: index
	"Look up the index-th entry of the directory with the given fully-qualified path 
	(i.e., starting from the root of the file hierarchy) and return an array containing:

	<name> <creationTime> <modificationTime> <dirFlag> <fileSize>

	The empty string enumerates the top-level files or drives. (For example, on Unix, the empty 
	path enumerates the contents of '/'. On Macs and PCs, it enumerates the mounted volumes/drives.)

	The creation and modification times are in seconds since the start of the Smalltalk time epoch. 
	DirFlag is true if the entry is a directory. FileSize the file size in bytes or zero for directories. 
	The primitive returns nil when index is past the end of the directory. It fails if the given path 
	is bad."

 	<primitive: 'primitiveDirectoryLookup' module: 'FilePlugin'>
	^ #badDirectoryPath

! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:27'!
open: fileName writable: writableFlag
	"Open a file of the given name, and return the file ID obtained.
	If writableFlag is true, then
		if there is none with this name, then create one
		else prepare to overwrite the existing from the beginning
	otherwise
		if the file exists, open it read-only
		else return nil"

	<primitive: 'primitiveFileOpen' module: 'FilePlugin'>
	^ nil
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:28'!
read: id into: byteArray startingAt: startIndex count: count
	"Read up to count bytes of data from this file into the given string or byte array starting at the given index. Answer the number of bytes actually read."

	<primitive: 'primitiveFileRead' module: 'FilePlugin'>
	self primitiveFailed! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:24'!
rename: oldFileFullName to: newFileFullName 
	"Rename the file of the given name to the new name. Fail if there is no file of the old name 
	or if there is an existing file with the new name."

	<primitive: 'primitiveFileRename' module: 'FilePlugin'>
	^nil! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:25'!
setMacFileNamed: fileName type: typeString creator: creatorString
	"Set the Macintosh file type and creator info for the file with the given name. Fails if the 
	file does not exist or if the type and creator type arguments are not strings of length 4. This
	primitive is Mac specific; it is a noop on other platforms."

 	<primitive: 'primitiveDirectorySetMacTypeAndCreator' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:30'!
setPosition: id to: anInteger
	"Set this file to the given position."

	<primitive: 'primitiveFileSetPosition' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:30'!
size: id
	"Answer the size of this file."

	<primitive: 'primitiveFileSize' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/22/2009 07:10'!
sizeOrNil: id
	"Answer the size of this file."

	<primitive: 'primitiveFileSize' module: 'FilePlugin'>
	^ nil! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:33'!
truncate: id to: anInteger
	"Truncate this file to the given position."

	<primitive: 'primitiveFileTruncate' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:33'!
write: id from: stringOrByteArray startingAt: startIndex count: count
	"Write count bytes onto this file from the given string or byte array starting at the given index. 	Answer the number of bytes written."

	<primitive: 'primitiveFileWrite' module: 'FilePlugin'>
	self primitiveFailed! !

FileSystemHandle subclass: #FileHandle
	instanceVariableNames: 'id'
	classVariableNames: 'Registry'
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!FileHandle commentStamp: 'cwp 11/18/2009 13:02' prior: 0!
I provide an interface for doing IO on an open file. I keep an id, which as an opaque identifier used by the FilePlugin primitives. I translate positions from the 1-based indexes used in Smalltalk to the 0-based offsets used by the primitives.

I do not implement the primitives myself, instead delegating those to an instance of FilePluginPrimitives.!

!FileHandle class methodsFor: 'class initialization' stamp: 'cwp 11/20/2009 16:16'!
initialize
	self useFilePlugin.
! !

!FileHandle class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 5/24/2012 15:06'!
register: aFileHandle
	"properly register the given FileHandle for being closed on garbage collection"
	^self registry add: aFileHandle! !

!FileHandle class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 5/24/2012 15:06'!
registry
	^Registry ifNil: [Registry := WeakRegistry new]
! !

!FileHandle class methodsFor: 'class initialization' stamp: 'cwp 11/20/2009 16:51'!
startUp: resuming
	"This functionality is disabled for now, to avoid doing a lot of processing at
	image start up. To reenable, add this class to the start up list."
	
	resuming ifTrue: [self allInstancesDo: [:ea | ea startUp]]! !

!FileHandle class methodsFor: 'class initialization' stamp: 'EstebanLorenzano 4/2/2012 11:46'!
useFilePlugin
	Primitives := FilePluginPrims new! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:19'!
assureOpen
	"compatibility method to make the FileHandle Tests pass"
	self isOpen ifFalse: [ id := self basicOpen ].! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:20'!
at: index read: buffer startingAt: start count: count
	
	self flag: 'TODO: remove once FileHandles are really used!!'.
	self assureOpen.
	
	^ Primitives 
		setPosition: id to: index - 1;
		read: id into: buffer startingAt: start count: count
! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:20'!
at: index write: buffer startingAt: start count: count
	
	self flag: 'TODO: remove once FileHandles are really used!!'.
	self assureOpen.
	
	Primitives 
		setPosition: id to: index - 1;
		write: id from: buffer startingAt: start count: count
! !

!FileHandle methodsFor: 'public' stamp: 'cwp 11/20/2009 14:59'!
close
	Primitives close: id.
	id := nil! !

!FileHandle methodsFor: 'finalization' stamp: 'CamilloBruni 5/24/2012 15:07'!
finalize
	self primCloseNoError: id.! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:20'!
flush
	
	self flag: 'TODO: remove once FileHandles are really used!!'.
	self assureOpen.
	
	Primitives flush: id! !

!FileHandle methodsFor: 'testing' stamp: 'cwp 7/22/2009 07:10'!
isOpen
	^ (Primitives sizeOrNil: id) notNil! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:18'!
open
	self flag: 'TODO: for now we solely rely on the old FileStreams'
	"id := self basicOpen.
	id ifNil: 	[
		reference exists ifFalse: [FileDoesNotExist signalWith: reference].
		self error: 'Unable to open file ' , reference printString]"! !

!FileHandle methodsFor: 'public' stamp: 'abc 5/11/2012 23:22'!
readStream
	^ (FileStream onHandle: self)
		ifNil: [ self streamError ]
	! !

!FileHandle methodsFor: 'finalization' stamp: 'CamilloBruni 5/24/2012 15:05'!
register
	"register the instance for proper clreanup on garbage collection"
	^self class register: self! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:20'!
size
	
	self flag: 'TODO: remove once FileHandles are really used!!'.
	self assureOpen.
	
	^ Primitives size: id! !

!FileHandle methodsFor: 'private' stamp: 'cwp 11/20/2009 16:48'!
startUp
	"This functionality is disabled for now, to avoid doing lots of processing
	on start up."
	
	"We're starting up in a new OS process, so the file id will be invalid.
	Try to reopen the file, but fail silently: just leave the id as nil. #isOpen will
	answer false, and we'll raise an error if anyone tries to do IO."
	
	self basicOpen! !

!FileHandle methodsFor: 'public' stamp: 'abc 5/11/2012 23:22'!
streamError
	reference exists
		ifFalse: [FileDoesNotExist signalWith: reference].
	self error: 'Unable to open file ' , reference printString! !

!FileHandle methodsFor: 'public' stamp: 'cwp 7/22/2009 08:17'!
truncateTo: anInteger
	Primitives truncate: id to: anInteger.
	self reopen! !

!FileHandle methodsFor: 'public' stamp: 'abc 5/11/2012 23:22'!
writeStream
	^( FileStream onHandle: self)
		 ifNil: [ self streamError ]! !
FileHandle initialize!
DiskStore initialize!
